//==============================================================================
//	File:      $URL: svn+ssh://repositorypub@repository.eecs.berkeley.edu/public/Projects/GateLib/branches/dev/Publications/Tutorials/Publications/EECS150/Labs/StructuralVerilog/Files/Mux21.v $
//	Version:   $Revision: 26904 $
//	Author:    Chris Fletcher (http://cwfletcher.net)
//	Copyright: Copyright 2009-2010 UC Berkeley
//==============================================================================

//==============================================================================
//	Section:	License
//==============================================================================
//	Copyright (c) 2005-2010, Regents of the University of California
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification,
//	are permitted provided that the following conditions are met:
//
//		- Redistributions of source code must retain the above copyright notice,
//			this list of conditions and the following disclaimer.
//		- Redistributions in binary form must reproduce the above copyright
//			notice, this list of conditions and the following disclaimer
//			in the documentation and/or other materials provided with the
//			distribution.
//		- Neither the name of the University of California, Berkeley nor the
//			names of its contributors may be used to endorse or promote
//			products derived from this software without specific prior
//			written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//	ANY EXPRESS OR InIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE InIMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//	ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//==============================================================================

//------------------------------------------------------------------------------
//	Module:  Mux21
//	Desc:    A 2:1 multiplexer implemented in Structural Verilog.
//	Inputs:  A: The first input.
//			B: The second input.
//            Select: Select bit.  If 1, B will be propogated to the output.
//			If 0, A will be propogated to the output.
//	Outputs: Out: The output.
//            
//	Author:  <a href="http://cwfletcher.net">Chris Fletcher</a>
//	Version: $Revision: 26904 $
//------------------------------------------------------------------------------
module Mux21(
			//------------------------------------------------------------------
			//	Input Interface
			//------------------------------------------------------------------
			A,
			B,
			Select,
			//------------------------------------------------------------------
			
			//------------------------------------------------------------------
			//	Output Interface
			//------------------------------------------------------------------
			Out
			//------------------------------------------------------------------
			);
	//--------------------------------------------------------------------------
	//	Inputs
	//--------------------------------------------------------------------------
	input wire										A, B, Select;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Outputs
	//--------------------------------------------------------------------------
	output wire										Out;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Wires
	//--------------------------------------------------------------------------
	wire											notSelect, AIntOut, BIntOut;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Structural 2:1 MUX
	//--------------------------------------------------------------------------
	not invertS(			notSelect, 				Select);
	and andAIntermediate(	AIntOut, 	A, 			notSelect),
		andBIntermediate(	BIntOut, 	B, 			Select);
	or 	result(				Out, 		AIntOut,	BIntOut);
	//--------------------------------------------------------------------------
endmodule
//------------------------------------------------------------------------------