//==============================================================================
//	File:     $URL: svn+ssh://repositorypub@repository.eecs.berkeley.edu/public/Projects/GateLib/branches/dev/Publications/Tutorials/Publications/EECS150/Labs/StructuralVerilog/Framework/TestHarnessAccumulator.v $
//	Version:  $Revision: 26904 $
//	Author:   Chris Fletcher (http://cwfletcher.net)
//	Copyright: Copyright 2009-2010 UC Berkeley
//==============================================================================

//==============================================================================
//	Section:	License
//==============================================================================
//	Copyright (c) 2005-2010, Regents of the University of California
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification,
//	are permitted provided that the following conditions are met:
//
//		- Redistributions of source code must retain the above copyright notice,
//			this list of conditions and the following disclaimer.
//		- Redistributions in binary form must reproduce the above copyright
//			notice, this list of conditions and the following disclaimer
//			in the documentation and/or other materials provided with the
//			distribution.
//		- Neither the name of the University of California, Berkeley nor the
//			names of its contributors may be used to endorse or promote
//			products derived from this software without specific prior
//			written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//	ANY EXPRESS OR InIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE InIMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//	ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//==============================================================================

//------------------------------------------------------------------------------
//	Module:	TestHarnessAccumulator
//	Desc:	The tester module for the Accumulator.
//
//	Author:	<a href="http://cwfletcher.net">Chris Fletcher</a>
//	Version:	$Revision: 26904 $
//------------------------------------------------------------------------------
module TestHarnessAccumulator(
			//------------------------------------------------------------------
			//	Clock and Related Signals
			//------------------------------------------------------------------
			Clock,
			Reset, 
			//------------------------------------------------------------------

			//------------------------------------------------------------------
			//	Input Interface
			//------------------------------------------------------------------
			ALUOp,
			//------------------------------------------------------------------
			
			//------------------------------------------------------------------
			//	Monitor Interface
			//------------------------------------------------------------------
			Success,
			Fail
			//------------------------------------------------------------------
	);
		
	//--------------------------------------------------------------------------
	//	Inputs
	//--------------------------------------------------------------------------
	input wire		[2:0]							ALUOp;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Parameters
	//--------------------------------------------------------------------------
	parameter		Width = 						1;				// DO NOT CHANGE this 'Width' value (only modify the 'Width' declared in FPGA_TOP_ML505 to change circuit width)
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Clock and Reset Inputs
	//--------------------------------------------------------------------------
	input wire										Clock, Reset;
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	Outputs
	//--------------------------------------------------------------------------
	output wire										Success, Fail;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Wires
	//--------------------------------------------------------------------------
	wire			[Width-1:0]						A;
	wire			[Width-1:0]						CUTResult, ExpectedResult;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	CUT
	//--------------------------------------------------------------------------
	Accumulator 	CUT (		.Clock(				Clock),
								.ClockEnable(		Success), 			// Only progress time if we have passed our current test
								.Reset(				Reset), 
								.Set(				1'b0),
								.A(					A),
								.ALUOp(				ALUOp),
								.Result(			CUTResult));
	defparam		CUT.Width =						Width;
	//--------------------------------------------------------------------------	
	
	//--------------------------------------------------------------------------
	//	Solution
	//--------------------------------------------------------------------------
	BehavioralAccumulator Solution (.Clock(			Clock),
								.ClockEnable(		Success), 
								.Reset(				Reset), 
								.Set(				1'b0),
								.A(					A),
								.ALUOp(				ALUOp),
								.Result(			ExpectedResult));
	defparam		Solution.Width =				Width;
	//--------------------------------------------------------------------------	

	//--------------------------------------------------------------------------
	//	Test Unit
	//--------------------------------------------------------------------------	
	Tester			TestUnit(	.Clock(				Clock),
								.Reset(				Reset), 
								.ResultA(			CUTResult),
								.ResultB(			ExpectedResult),
								.Count(				A),
								.Success(			Success),
								.Fail(				Fail));
	defparam		TestUnit.CWidth = 				Width;
	defparam		TestUnit.Width = 				Width;
	//--------------------------------------------------------------------------
endmodule
//------------------------------------------------------------------------------