//==============================================================================
//	File:     $URL: svn+ssh://repositorypub@repository.eecs.berkeley.edu/public/Projects/GateLib/branches/dev/Publications/Tutorials/Publications/EECS150/Labs/StructuralVerilog/Framework/Tester.v $
//	Version:  $Revision: 26904 $
//	Author:   Chris Fletcher (http://cwfletcher.net)
//	Copyright: Copyright 2009-2010 UC Berkeley
//==============================================================================

//==============================================================================
//	Section:	License
//==============================================================================
//	Copyright (c) 2005-2010, Regents of the University of California
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification,
//	are permitted provided that the following conditions are met:
//
//		- Redistributions of source code must retain the above copyright notice,
//			this list of conditions and the following disclaimer.
//		- Redistributions in binary form must reproduce the above copyright
//			notice, this list of conditions and the following disclaimer
//			in the documentation and/or other materials provided with the
//			distribution.
//		- Neither the name of the University of California, Berkeley nor the
//			names of its contributors may be used to endorse or promote
//			products derived from this software without specific prior
//			written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//	ANY EXPRESS OR InIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE InIMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//	ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//==============================================================================

//------------------------------------------------------------------------------
//	Module:	Tester
//	Desc:	Compares two results and increments a count if they succeed.  If the
//			results show a mismatch, the count stalls and the appropriate flag
//			(Success/Fail) is raised.  Since the Count saturates, it will stop
//			when it has reached its maximum value.
//
//	Author:	<a href="http://cwfletcher.net">Chris Fletcher</a>
//	Version:	$Revision: 26904 $
//------------------------------------------------------------------------------
module Tester(
			//------------------------------------------------------------------
			//	Clock and Related Signals
			//------------------------------------------------------------------
			Clock,
			Reset, 
			//------------------------------------------------------------------
			
			//------------------------------------------------------------------
			//	Input Interface
			//------------------------------------------------------------------
			ResultA,
			ResultB,
			//------------------------------------------------------------------
			
			//------------------------------------------------------------------
			//	Output Interface
			//------------------------------------------------------------------
			Count,
			Success,
			Fail
			//------------------------------------------------------------------
	);
	//--------------------------------------------------------------------------
	//	Parameters
	//--------------------------------------------------------------------------
	parameter		Width = 						2,				// The width of the input Result lines (DO NOT CHANGE)
					CWidth = 						Width;			// The width of the Count output (DO NOT CHANGE)
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Clock and Reset Inputs
	//--------------------------------------------------------------------------
	input wire										Clock, Reset;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Inputs
	//--------------------------------------------------------------------------
	input wire		[Width-1:0]						ResultA, ResultB;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Outputs
	//--------------------------------------------------------------------------
	output reg		[CWidth-1:0]					Count;
	output wire										Success, Fail;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Output Assigns
	//--------------------------------------------------------------------------
	assign			Success =						ResultA == ResultB;
	assign			Fail = 							~Success;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Counter
	//--------------------------------------------------------------------------
	always@(posedge Clock) begin
		if (Reset) 	Count <=						{CWidth{1'b0}};
		else if ((Count != {CWidth{1'b1}}) & Success) Count <= Count + 1'b1;	
	end
	//--------------------------------------------------------------------------	
endmodule
//------------------------------------------------------------------------------